// ****************************************************************
//
//   IBM Confidential
//   OCO Source Materials
//   <<PART NUMBER - 5724-D15>>
//   (C) Copyright IBM Corp. 2001, 2004 - All Rights Reserved.
//   The source code for this program is not published or otherwise divested
//   of its trade secrets, irrespective of what has been deposited with the
//   U.S. Copyright Office.
//
// ****************************************************************
package com.ibm.etools.marshall.util;

import java.math.BigInteger;

/**
 * This class defines the behavior for a range of BigInteger values.  This
 * class is intended to be used with the possible value construct
 * on a field.
 */

public class BigIntegerRange implements IRangeObject
{
	private static String copyright = "(c) Copyright IBM Corporation 2000, 2004.";
	private static final long serialVersionUID = 5L;
	
	protected BigInteger lowerRange_;
	protected BigInteger upperRange_;		

/**
 * Allocates a new BigIntegerRange with a lower and upper range value of 0.
 */

public BigIntegerRange()
{
	lowerRange_ = new BigInteger("0");
	upperRange_ = new BigInteger("0");
}


/**
 * Creates a new instance of a BigIntegerRange object with the specified upper
 * and lower ranges.
 * @param lowerRange The lower range.
 * @param upperRange The upper range.
 * @exception java.lang.IllegalArgumentException The upper range is less than
 * the lower range.
 */

public BigIntegerRange(BigInteger lowerRange, BigInteger upperRange) throws IllegalArgumentException 
{
	if (lowerRange.compareTo(upperRange) == 1)
		throw new IllegalArgumentException(upperRange.toString());

	lowerRange_ = lowerRange;
	upperRange_ = upperRange;
}


/**
 * Creates a new object of the same class as this object. It then initializes
 * each of the new object's fields by assigning it the same value as
 * the corresponding field in this object. No constructor is called.
 * @return A new object initialized to the same state as this object.
 */
 
public Object clone()
{
	try
	{
		BigIntegerRange newType = (BigIntegerRange)super.clone();

		return (newType);
	}
	catch (CloneNotSupportedException ex)
	{
		throw new java.lang.InternalError(ex.getMessage());
	}
}


/**
 * This method determines if the two BigInteger range objects are equivalent.
 * @return <code>true</code> if this range object is the same as the obj
 * argument; <code>false</code> otherwise.
 * @param obj The reference object with which to compare.
 */

public boolean equals (Object obj)
{
	if (obj == null)
		return (false);

	if (this == obj)
		return (true);

	if (!(obj instanceof BigIntegerRange))
		return (false);

	BigIntegerRange tmpRange = (BigIntegerRange)obj;

	if (lowerRange_ != tmpRange.lowerRange_)
		return (false);

	if (upperRange_ != tmpRange.upperRange_)
		return (false);

	return (true);
}


/**
 * Returns the lower range, or bounds, of the range object as a
 * java.math.BigInteger object.
 * @return The lower range, or bounds, of the range object.
 */

public Object getLowerRange()
{
	return (lowerRange_);
}


/**
 * Returns the upper range, or bounds, of the range object as a
 * java.math.BigInteger object.
 * @return The upper range, or bounds, of the range object.
 */

public Object getUpperRange()
{
	return (upperRange_);
}


/**
 * Tests whether the specified value lies within the upper and lower range
 * set on the range object.
 * @return <code>true</code> if the specified value lies within the upper
 * and lower range; <code>false</code> otherwise.
 * @param value The value to compare against.  Must be a java.math.BigInteger object.
 */

public boolean isInRange (Object value)
{
	if (!(value instanceof BigInteger))
		return (false);

	return (isInRange (lowerRange_, upperRange_, (BigInteger)value));
}


/**
 * Tests whether the specified value lies within the specifed upper and lower
 * range.
 * @return <code>true</code> if the specified value lies within the upper
 * and lower range; <code>false</code> otherwise.
 * @param lowerRange The lower range limit.
 * @param upperRange The upper range limit.
 * @param value The value to compare against.
 * @exception java.lang.IllegalArgumentException The upper range is less than
 * the lower range.
 */

public static boolean isInRange(BigInteger lowerRange, BigInteger upperRange, BigInteger value) 
	throws IllegalArgumentException 
{
	if (lowerRange.compareTo(upperRange) == 1)
		throw new IllegalArgumentException(upperRange.toString());

	if ((value.compareTo(lowerRange) == -1) || (value.compareTo(upperRange) == 1))
		return (false);

	return (true);	
}


/**
 * Sets the lower range, or bounds, of the range object.
 * @param value The lower range value.  Must be a java.math.BigInteger object.
 * @exception IllegalArgumentException The value specified is not valid for
 * this range object.
 */

public void setLowerRange(Object value) throws IllegalArgumentException 
{
	if (!(value instanceof BigInteger))
		throw new IllegalArgumentException(value.toString());

	lowerRange_ = (BigInteger)value;

	return;	
}


/**
 * Sets the upper range, or bounds, of the range object.
 * @param value The upper range value.  Must be a java.math.BigInteger object.
 * @exception IllegalArgumentException The value specified is not valid for
 * this range object.
 */

public void setUpperRange (Object value) throws IllegalArgumentException 
{
	if (!(value instanceof BigInteger))
		throw new IllegalArgumentException(value.toString());

	upperRange_ = (BigInteger)value;

	return;	
}
}