// ****************************************************************
//
//   IBM Confidential
//   OCO Source Materials
//   <<PART NUMBER - 5724-B75>>
//   (C) Copyright IBM Corp. 2001 - All Rights Reserved.
//   US Government Users Restricted Rights - Use, duplication or disclosure
//   restricted by GSA ADP Schedule Contract with IBM Corp.
//
// ****************************************************************

package com.ibm.etools.marshall.util;

import java.math.BigDecimal;

public class MarshallFloatUtils
{
	private static final String copyright = "(c) Copyright IBM Corporation 2001.";


// Floating point formats

public final static int FLOAT_FORMAT_UNSPECIFIED = 0;
public final static int FLOAT_FORMAT_IEEE_EXTENTED_INTEL = 1;
public final static int FLOAT_FORMAT_IEEE_EXTENTED_AIX = 2;
public final static int FLOAT_FORMAT_IEEE_EXTENTED_OS390 = 3;
public final static int FLOAT_FORMAT_IEEE_EXTENTED_AS400 = 4;
public final static int FLOAT_FORMAT_IEEE_NON_EXTENTED = 5;
public final static int FLOAT_FORMAT_IBM_390_HEX = 6;
public final static int FLOAT_FORMAT_IBM_400_HEX = 7;


static public void marshallFloatIntoBuffer (BigDecimal value, byte[] buffer, int startingOffset, boolean isBigEndian, int floatType, int size)
	throws IllegalArgumentException
{
	switch (size)
	{
		case 4:
			float floatValue = value.floatValue();
			marshallFloatIntoBuffer (floatValue, buffer, startingOffset, isBigEndian, floatType, size);
		break;


		case 8:
			double doubleValue = value.doubleValue();
			marshallFloatIntoBuffer (doubleValue, buffer, startingOffset, isBigEndian, floatType, size);
		break;

		case 16:
			byte[] tb = null;

			switch (floatType)
			{
				case FLOAT_FORMAT_IEEE_EXTENTED_INTEL:
					tb = ConversionUtils.convertBigDecimalToIEEEExtendedIntel (value);
					break;

				case FLOAT_FORMAT_IEEE_EXTENTED_AIX:
					tb = ConversionUtils.convertBigDecimalToIEEEExtendedAIX (value);
					break;

				case FLOAT_FORMAT_IEEE_EXTENTED_OS390:
					tb = ConversionUtils.convertBigDecimalToIEEEExtendedOS390 (value);
					break;

				case FLOAT_FORMAT_IBM_390_HEX:
				case FLOAT_FORMAT_IBM_400_HEX:
					tb = ConversionUtils.convertBigDecimalToIBM390HexExtended (value);
					break;

				default:
					throw new IllegalArgumentException(MarshallResource.instance().getString(MarshallResource.MARSHRT_FP_16_NOT_SUPPORTED));
			}

			if (!(isBigEndian))
				ConversionUtils.endianSwap (tb);
				
			System.arraycopy (tb, 0, buffer, startingOffset, tb.length);
			break;

		default:
			throw new IllegalArgumentException(MarshallResource.instance().getString(MarshallResource.MARSHRT_FP_SIZE_NOT_VALID, Integer.toString(size)));
	}

	return;
}

static public void marshallFloatIntoBuffer (double value, byte[] buffer, int startingOffset, boolean isBigEndian, int floatType, int size)
	throws IllegalArgumentException
{
	byte[] tb = null;

	if (size != 8)
		throw new IllegalArgumentException(MarshallResource.instance().getString(MarshallResource.MARSHRT_FP_SIZE_NOT_VALID, Integer.toString(size)));

	if ((floatType == FLOAT_FORMAT_IBM_390_HEX) || (floatType == FLOAT_FORMAT_IBM_400_HEX))
	{
		try
		{
			long s390Value = ConversionUtils.doubleToS390LongBits (value);
			tb = ConversionUtils.convertToByte (s390Value);
		}
		catch (java.io.IOException exc)
		{
			throw new IllegalArgumentException (exc.getMessage());
		}
	}
	else
		tb = ConversionUtils.convertToByte (value);

	if (!(isBigEndian))
		ConversionUtils.endianSwap (tb);
		
	System.arraycopy (tb, 0, buffer, startingOffset, tb.length);

	return;
}


static public void marshallFloatIntoBuffer (float value, byte[] buffer, int startingOffset, boolean isBigEndian, int floatType, int size)
	throws IllegalArgumentException
{
	byte[] tb = null;

	if (size != 4)
		throw new IllegalArgumentException(MarshallResource.instance().getString(MarshallResource.MARSHRT_FP_SIZE_NOT_VALID, Integer.toString(size)));

	if ((floatType == FLOAT_FORMAT_IBM_390_HEX) || (floatType == FLOAT_FORMAT_IBM_400_HEX))
	{
		try
		{
			int s390Value = ConversionUtils.floatToS390IntBits (value);
			tb = ConversionUtils.convertToByte (s390Value);
		}
		catch (java.io.IOException exc)
		{
			throw new IllegalArgumentException (exc.getMessage());
		}
	}
	else
		tb = ConversionUtils.convertToByte (value);

	if (!(isBigEndian))
		ConversionUtils.endianSwap (tb);
		
	System.arraycopy (tb, 0, buffer, startingOffset, tb.length);

	return;
}


static public BigDecimal unmarshallBigDecimalFromBuffer (byte[] buffer, int startingOffset, boolean isBigEndian, int floatType, int size)
	throws IllegalArgumentException
{
	BigDecimal result = null;

	switch (size)
	{
		case 4:
			float floatResult = unmarshallFloatFromBuffer (buffer, startingOffset, isBigEndian, floatType, size);
			result = new BigDecimal(Float.toString(floatResult));
		break;

		case 8:
			double doubleResult = unmarshallDoubleFromBuffer (buffer, startingOffset, isBigEndian, floatType, size);
			result = new BigDecimal(Double.toString(doubleResult));
		break;

		case 16:
			byte[] sb = null;

			if (floatType == FLOAT_FORMAT_IEEE_EXTENTED_INTEL)
				sb = new byte[10];
			else
				sb = new byte[16];
			
			System.arraycopy (buffer, startingOffset, sb, 0, sb.length);
		
			if (!(isBigEndian))
				ConversionUtils.endianSwap (sb);

			String fpString = null;

			switch (floatType)
			{
				case FLOAT_FORMAT_IEEE_EXTENTED_INTEL:
					fpString = ConversionUtils.convertIEEEExtendedIntelToString (sb);
					break;

				case FLOAT_FORMAT_IEEE_EXTENTED_AIX:
					fpString = ConversionUtils.convertIEEEExtendedAIXToString (sb);
					break;

				case FLOAT_FORMAT_IEEE_EXTENTED_OS390:
					fpString = ConversionUtils.convertIEEEExtendedOS390ToString (sb);
					break;

				case FLOAT_FORMAT_IBM_390_HEX:
				case FLOAT_FORMAT_IBM_400_HEX:
					fpString = ConversionUtils.convertIBM390HexExtendedToString (sb);
					break;

				default:
					throw new IllegalArgumentException(MarshallResource.instance().getString(MarshallResource.MARSHRT_FP_16_NOT_SUPPORTED));
			}

			result = new BigDecimal(fpString);

			break;

		default:
			throw new IllegalArgumentException(MarshallResource.instance().getString(MarshallResource.MARSHRT_FP_SIZE_NOT_VALID, Integer.toString(size)));
	}

	return (result);
}

static public double unmarshallDoubleFromBuffer (byte[] buffer, int startingOffset, boolean isBigEndian, int floatType, int size)
	throws IllegalArgumentException
{
	if (size != 8)
		throw new IllegalArgumentException(MarshallResource.instance().getString(MarshallResource.MARSHRT_FP_SIZE_NOT_VALID, Integer.toString(size)));

	byte[] sb = new byte[8];
	
	System.arraycopy (buffer, startingOffset, sb, 0, sb.length);

	if (!(isBigEndian))
		ConversionUtils.endianSwap (sb);

	double result = 0.0;

	if ((floatType == FLOAT_FORMAT_IBM_390_HEX) || (floatType == FLOAT_FORMAT_IBM_400_HEX))
	{
		try
		{
			result = ConversionUtils.longS390BitsToDouble (ConversionUtils.convertFromByteToLong(sb));
		}
		catch (java.io.IOException exc)
		{
			throw new IllegalArgumentException(exc.getMessage());
		}
	}
	else
		result = ConversionUtils.convertFromByteToDouble (sb);

	return (result);
}


static public float unmarshallFloatFromBuffer (byte[] buffer, int startingOffset, boolean isBigEndian, int floatType, int size)
	throws IllegalArgumentException
{
	if (size != 4)
		throw new IllegalArgumentException(MarshallResource.instance().getString(MarshallResource.MARSHRT_FP_SIZE_NOT_VALID, Integer.toString(size)));

	byte[] sb = new byte[4];
	
	System.arraycopy (buffer, startingOffset, sb, 0, sb.length);

	if (!(isBigEndian))
		ConversionUtils.endianSwap (sb);

	float result = 0;

	if ((floatType == FLOAT_FORMAT_IBM_390_HEX) || (floatType == FLOAT_FORMAT_IBM_400_HEX))
	{
		try
		{
			result = ConversionUtils.intS390BitsToFloat (ConversionUtils.convertFromByteToInt(sb));
		}
		catch (java.io.IOException exc)
		{
			throw new IllegalArgumentException(exc.getMessage());
		}
	}
	else
		result = ConversionUtils.convertFromByteToFloat (sb);

	return (result);
}

}